#!/usr/local/bin/python3

"""

  Manage an incoming request for a barycentric radial velocity calculation.   

"""  

# John Kielkopf
# kielkopf at louisville.edu


# System dependencies


# This line ensures that astropy will reuse cache files in a server environment 
# If you are running as a user with a home directory, comment out the following two lines

import os

os.environ["XDG_CONFIG_HOME"] = "/data1/www/cgi--bin/barycorr/data/"

# For web use option for debugging uncomment the next two lines

import cgitb
cgitb.enable() 

# Python dependencies

import urllib                            # for leapsecond support
import cgi                               # for CGI

import numpy as np                       # for managing the data
import re                                # for manipulating strings
import math
from time import gmtime, strftime, time  # for utc
import datetime

from astropy.time import Time
from barycorrpy import get_BC_vel
from barycorrpy import JDUTC_to_BJDTDB

# Flags

test_barycorr_flag = False
diagnostics_flag = False
html_test_flag = False

def html_test():
  print ("Content-Type: text/html")
  print ("")
  
  print ("<br />")
  print ("<b>Test request for barycentric corrections</b>")
  print ("<br />")  


# Uploaded strings and values used
# The submit and opts fields will be ignored by this program.
 

jd_list_str = ""
jd_list = []
ra_str = "0.0"
ra_deg = 0.0
dec_str = "0.0"
dec_deg = 0.0
latitude_str = "0.0"
latitude_deg = 0.0
longitude_str = "0.0"
longitude_deg = 0.
elevation_str = "0.0"
elevation_m = 0.0
pmra_str = "0.0"
pmra_mas = 0.0
pmdec_str = "0.0"
pmdec_mas = 0.0
parallax_str = "0.0"
parallax_mas = 0.
rv_str = "0.0"
rv_kmps = 0.0
zmeasure_str = "0.0"
zmeasure = 0.0
epoch_str = "2451545.0"
epoch_jd = 2451545.0
jd_base_str = "0.0"
jd_base = 0.0
jd_list = []

# Define test values for Tau Ceti

def test_inputs():
  ra_deg =  26.0213645867
  dec_deg = -15.9395557246
  latitude_deg = -30.169283
  longitude_deg = -70.806789
  elevation_meters = 2241.9
  epoch_jd = 2451545.0
  pmra_mas = -1721.05
  pmdec_mas = 854.16 
  parallax_mas = 273.96
  rv_kmps = 0.0
  z_measure = 0.0
  ephemeris = "de430"
  leap_update = True
  global jd_list
  
  # Input JDs can be a list of floats or an Astropy Time Object
  
  jd_list = [2458000.0, 2458000.00001, 2458000.00002]
  
  # Arguments of get_BC_vel could include 
  #   ephemeris=ephemeris
  #   leap_update=leap_update 
  
  results = get_BC_vel(JDUTC=jd_list, ra=ra_deg, dec=dec_deg,
    lat=latitude_deg, longi=longitude_deg, alt=elevation_m,
    pmra = pmra_mas, pmdec=pmdec_mas, px=parallax_mas,
    rv = rv_kmps, zmeas=z_measure, epoch=epoch_jd)
  
  # Expected values: [15407.486, 15407.4723, 15407.4586]

          
  return results[0]
  

# If testing only barycorr simply run barycorr with coded values, return answers, and exit

if test_barycorr_flag:
    
  corrections = test_inputs()
  print ("Content-Type: text/html")
  print ("")
  print ("<br />")
  print ("<br />")
  for n in range(len(corrections)):
    print(jd_list[n], "  ", corrections[n])
    print ("<br />")
  print ("<br />")  
  exit()


# Check for validitiy of RA and Dec strings
# Parse coordinate strings into degrees for center of search

def parse_coords():
  
  global ra_deg
  global dec_deg
  
  rahr = 0.0
  ramin = 0.0
  rasec = 0.0
  decdeg = 0.0
  decmin = 0.0
  decsec = 0.0

  # Test for empty strings

  if (ra_str == ""):
    print ("Content-Type: text/html")
    print ("")

    print ("<br />")
    print ("<b>Missing a right ascension</b>")
    print ("<br />")
    print ("<br />")
    exit()

  if (dec_str == ""):
    print ("Content-Type: text/html")
    print ("")

    print ("<br />")
    print ("<b>Missing a declination</b>")
    print ("<br />")
    print ("<br />")
    exit()

  # Treat RA based on content 
  # If RA has : then it is hexasegismal hours and otherwise it is decimal degrees
  
  if ':' in ra_str:
    rahrstr, raminstr, rasecstr = ra_str.split(":")
    rahr = abs(float(rahrstr))
    ramin = abs(float(raminstr))
    rasec = abs(float(rasecstr))
    if float(rahrstr) < 0:
      rasign = -1.
    else:
      rasign = +1.
    ra_deg = rasign*(rahr + ramin/60. + rasec/3600.)*15.
  else:  
    try:
      ra_deg = float(ra_str)
    except:
      print ("Content-Type: text/html")
      print ("")

      print ("<br />")
      print ("<b>Cannot parse right ascension entry.</b>")
      print ("<br />")
      print ("<br />")
      exit()    
       

  if ':' in dec_str:
    decdegstr, decminstr, decsecstr = dec_str.split(":")
    decdeg = abs(float(decdegstr))
    decmin = abs(float(decminstr))
    decsec = abs(float(decsecstr))
    if float(decdegstr) < 0:
      decsign = -1.
    elif decdegstr[0] == "-":
      decsign = -1.  
    else:
      decsign = +1.
    dec_deg = decsign*(decdeg + decmin/60. + decsec/3600.)
  else:  
    try:
      dec_deg = float(dec_str) 
    except:
      print ("Content-Type: text/html")
      print ("")

      print ("<br />")
      print ("<b>Cannot parse declination entry.</b>")
      print ("<br />")
      print ("<br />")
      exit()
  return()          




# Parse the cgi fields using these keys matching those from Jason Eastman's retired website

# 'JDS'               comma separated list of JDs to be added to TBASE 
# 'RA'                hh:mm:ss or float degrees   
# 'DEC'               dd:mm:ss or float degrees  
# 'LAT'               degrees defaults to Minerva Australis
# 'LON'               degrees + east  defaults to Minerva Australis
# 'ELEVATION'         meters  
# 'PMRA'              milli-arcseconds per year  
# 'PMDEC'             milli-arcseconds per year  
# 'PARALLAX'          milli-arcseconds  
# 'RV'                km/s           
# 'ZMEAS'             v/c  
# 'EPOCH'             JD  
# 'TBASE'             base JD to which JDs are added

          
request = cgi.FieldStorage()

jd_str = request.getfirst("JDS", "2451545.0")
ra_str = request.getfirst("RA", "")
dec_str =  request.getfirst("DEC", "")
latitude_str = request.getfirst("LAT", "0.0")
longitude_str = request.getfirst("LON", "0.0")
elevation_str = request.getfirst("ELEVATION", "0.0")
pmra_str = request.getfirst("PMRA", "0.0")
pmdec_str = request.getfirst("PMDEC", "0.0")
parallax_str = request.getfirst("PARALLAX", "0.0")
rv_str = request.getfirst("RV", "0.0")
zmeasure_str = request.getfirst("ZMEAS", "0.0")
epoch_str = request.getfirst("EPOCH", "2451545.0")
jd_base_str = request.getfirst("TBASE", "0.0") 



# Examples of string processing
#
# Remove all but numbers and dashes from date-obs
#
# date_obs_str = re.sub(r'([^-0-9])+', '', date_obs_str)
# if date_obs_str =="":
#   date_obs_str = "2019-01-01"
#
# Remove non-numeric characters and spaces 
#
# magnitude_str = float(re.sub(r'([^-+\.0-9])+', '', magnitude_str))
# depth_str = float(re.sub(r'([^-+\.0-9])+', '', depth_str))
#
# Remove unacceptable characters and trailing spaces 
#
# ra_str = re.sub(r'([^-+\.\:0-9])+', '', ra_str).strip()
# dec_str = re.sub(r'([^-+\.\:0-9])+', '', dec_str).strip()

if html_test_flag:
  html_test()

# Check and respond to missing necessary fields
 
if (ra_str == "" or dec_str == ""):
  print ("Content-Type: text/html")
  print ("")

  print ("<br />")
  print ("<b>Missing coordinates from your submission</b>")
  print ("<br />")
  print ("<br />")
  exit()

# Set the fields for input to barycorr

# Coordinates

ra_str = re.sub(r'([^-+\.\:0-9])+', '', ra_str).strip()
dec_str = re.sub(r'([^-+\.\:0-9])+', '', dec_str).strip()
parse_coords()

pmra_str = re.sub(r'([^-+\.\:0-9])+', '', pmra_str).strip()
pmdec_str = re.sub(r'([^-+\.\:0-9])+', '', pmdec_str).strip()
parallax_str = re.sub(r'([^-+\.\:0-9])+', '', parallax_str).strip()

pmra_mas = float(pmra_str)
pmdec_mas = float(pmdec_str)
parallax_mas = float(parallax_str)


# Site defaults to MKO

if (latitude_str != ""):
  latitude_str = re.sub(r'([^-+\.\:0-9])+', '', latitude_str).strip()
  latitude_deg = float(latitude_str)
    
if (longitude_str != ""):
  longitude_str = re.sub(r'([^-+\.\:0-9])+', '', longitude_str).strip()
  longitude_deg = float(longitude_str)  

if (elevation_str != ""):
  elevation_str = re.sub(r'([^-+\.\:0-9])+', '', elevation_str).strip()
  elevation_m = float(elevation_str)

# Velocities

rv_str = re.sub(r'([^-+\.\:0-9])+', '', rv_str).strip()
zmeasure_str = re.sub(r'([^-+\.\:0-9])+', '', zmeasure_str).strip()
rv_kmps = float(rv_str)
zmeasure = float(zmeasure_str)

# Dates

epoch_str = re.sub(r'([^-+\.\:0-9])+', '', epoch_str).strip()
epoch = float(epoch_str)

jd_base_str = re.sub(r'([^-+\.\:0-9])+', '', jd_base_str).strip()
jd_base = float(jd_base_str)
jd_str = re.sub(r'([^-+\.\:0-9\,])+', '', jd_str).strip().split(",")

for jd_item in jd_str:  
  jd = float(jd_item)  
  jd = jd + jd_base
  jd_list.append(jd)


bc_rvs = get_BC_vel(JDUTC=jd_list,
  ra = ra_deg,
  dec = dec_deg,
  lat = latitude_deg,
  longi = longitude_deg,
  alt = elevation_m,
  pmra = pmra_mas,
  pmdec = pmdec_mas,
  px = parallax_mas,
  rv = rv_kmps,
  zmeas = zmeasure,
  epoch = epoch_jd,
  ephemeris = "de430")

tdbs = JDUTC_to_BJDTDB(JDUTC=jd_list,
  ra = ra_deg,
  dec = dec_deg,
  lat = latitude_deg,
  longi = longitude_deg,
  alt = elevation_m,
  pmra = pmra_mas,
  pmdec = pmdec_mas,
  px = parallax_mas,
  rv = rv_kmps,
  epoch = epoch,
  ephemeris = "de430")


# The input request has been parsed
# Now acknowledge the request

print ("Content-Type: text/html")
print ("")

print ("<br />")
print ("<b>Request sent for barycentric corrections for this target</b>")
print ("<br />")
print ("<br />")
print ("RA = ", ra_str)
print ("<br />")
print ("Dec = ", dec_str)
print ("<br />")
print ("<br />")

if diagnostics_flag:

  # Print enough to test for proper reading and inputs to BC_vel
  
  print("JDs: ",jd_str, jd_list, "<br>")
  print("Latitude: ", latitude_str, latitude_deg, "<br>")
  print("Longitude: ", longitude_str, longitude_deg, "<br>")
  print("Elevation: ", elevation_str, elevation_m, "<br>")
  print("Proper motion RA: ", pmra_str, pmra_mas, "<br>")
  print("Proper motion Dec: ", pmdec_str, pmdec_mas, "<br>")
  print("Parallax: ", parallax_str, parallax_mas, "<br>")
  print("RV: ", rv_str, rv_kmps, "<br>")
  print("Z: ", zmeasure_str, zmeasure, "<br>")
  print("Epoch: ", epoch_str, epoch_jd, "<br>")
  print ("<br />")
  print ("<br />")

print ("<br />")
print ("<br />")

print("<table width=\"400\" border=\"0\">")
print("<tr>")
print("<td> JDUTC </td>")
print("<td> BJDTDB </td>")
print("<td> BCRV (m/s) </td>")
print("</tr>")
for n in range(len(bc_rvs[0])):
  print("<tr>")
  print("<td>", "{:16.8f}".format(jd_list[n]), "</td>")
  print("<td>", "{:16.8f}".format(tdbs[0][n]), "</td>")
  print("<td>", "{:9.3f}".format(bc_rvs[0][n]), "</td>")
  print("</tr>")
print("</table>")  
exit()












